<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       www.skrotron.com
 * @since      1.0.7
 *
 * @package    Cart_Upsell_For_Woocommerce
 * @subpackage Cart_Upsell_For_Woocommerce/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.7
 * @package    Cart_Upsell_For_Woocommerce
 * @subpackage Cart_Upsell_For_Woocommerce/includes
 */
class Cart_Upsell_For_Woocommerce {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.7
	 * @var      Cart_Upsell_For_Woocommerce_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.7
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.7
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.7
	 */
	public function __construct() {
		if ( defined( 'CART_UPSELL_FOR_WOOCOMMERCE_VERSION' ) ) {
			$this->version = CART_UPSELL_FOR_WOOCOMMERCE_VERSION;
		} else {
			$this->version = '1.0.7';
		}
		$this->plugin_name = 'cart-upsell-for-woocommerce';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cart-upsell-functions.php';

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Cart_Upsell_For_Woocommerce_Loader. Orchestrates the hooks of the plugin.
	 * - Cart_Upsell_For_Woocommerce_i18n. Defines internationalization functionality.
	 * - Cart_Upsell_For_Woocommerce_Admin. Defines all hooks for the admin area.
	 * - Cart_Upsell_For_Woocommerce_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.7
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cart-upsell-for-woocommerce-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cart-upsell-for-woocommerce-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-cart-upsell-for-woocommerce-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-cart-upsell-for-woocommerce-public.php';

		$this->loader = new Cart_Upsell_For_Woocommerce_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Cart_Upsell_For_Woocommerce_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.7
	 */
	private function set_locale() {

		$plugin_i18n = new Cart_Upsell_For_Woocommerce_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.7
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Cart_Upsell_For_Woocommerce_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'skrotron_enqueue_color_picker_for_cart_upsell' );
		$this->loader->add_filter( 'product_type_options', $plugin_admin, 'add_cart_upsell_option' );
		$this->loader->add_filter( 'woocommerce_product_data_tabs', $plugin_admin, 'cart_upsell_tabs' );
		$this->loader->add_action( 'woocommerce_product_data_panels', $plugin_admin, 'cart_upsell_tab_content' );
		$this->loader->add_action( 'woocommerce_process_product_meta_simple', $plugin_admin, 'save_cart_upsell_data_on_update' );
		$this->loader->add_action( 'wp_ajax_nopriv_cart_upsell_get_dropdown_content', $plugin_admin, 'cart_upsell_get_dropdown_content' );
		$this->loader->add_action( 'wp_ajax_cart_upsell_get_dropdown_content', $plugin_admin, 'cart_upsell_get_dropdown_content' );
		$this->loader->add_action( 'wp_ajax_nopriv_cart_upsell_save_conditions', $plugin_admin, 'cart_upsell_save_conditions' );
		$this->loader->add_action( 'wp_ajax_cart_upsell_save_conditions', $plugin_admin, 'cart_upsell_save_conditions' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.7
	 */
	private function define_public_hooks() {

		$plugin_public = new Cart_Upsell_For_Woocommerce_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_action( 'woocommerce_before_cart_table', $plugin_public, 'cart_upsell_on_cart_page' );
		$this->loader->add_action( 'wp_ajax_cart_upsell_ajax_add_to_cart', $plugin_public, 'cart_upsell_ajax_add_to_cart' );
		$this->loader->add_action( 'wp_ajax_nopriv_cart_upsell_ajax_add_to_cart', $plugin_public, 'cart_upsell_ajax_add_to_cart' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.7
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.7
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.7
	 * @return    Cart_Upsell_For_Woocommerce_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.7
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
